/*
    ServicesCtl is a programm to start and stop Services from the TrayBar.
    Copyright (C) 2007  Olaf Groeger

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    For any feedback contact me at servicesctl@olaf-groeger.de.
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Reflection;
using System.Resources;
using System.ServiceProcess;
using System.Text;
using System.Windows.Forms;

using log4net;

namespace ServicesCtl
{
    /// <summary>
    /// This class controls the applications UI.
    /// </summary>
    class Controller
    {
        /// <summary>
        /// A static logger.
        /// </summary>
        private static readonly ILog log = LogManager.GetLogger(typeof(Controller));

        /// <summary>
        /// A private System.Resources.ResourceManager for handling the internationalization.
        /// </summary>
        private ResourceManager messages = null;

        /// <summary>
        /// The trayBar icon.
        /// </summary>
        private NotifyIcon notifyicon;
        /// <summary>
        /// The context menu show for Controller.notifyicon.
        /// </summary>
        private ContextMenu menu;
        /// <summary>
        /// The list of the services to be shown in the Controller.menu.
        /// </summary>
        private ArrayList controlledServices;
        /// <summary>
        /// The Administrator object, used for Impersonation.
        /// </summary>
        private Administrator administrator;

        /// <summary>
        /// Creates a new instance.
        /// </summary>
        /// <remarks>Initializes the ResourceManager Controller.messages, 
        /// the Administrator object Controller.administrator and adds an empty
        /// context menu to the Controller.notifyicon.
        /// </remarks>
        public Controller()
        {
            this.messages = new ResourceManager("ServicesCtl.Messages", Assembly.GetExecutingAssembly());

            notifyicon = new NotifyIcon();
            notifyicon.Visible = true;
            notifyicon.Icon = new System.Drawing.Icon(GetType(), @"appicon.ico");

            notifyicon.Text = this.messages.GetString("Controller_NotifyIcon_Text");

            menu = new ContextMenu(); //ContextMenu(menuItems);
            menu.Popup += notifyIcon_Popup;
            notifyicon.ContextMenu = menu;

            this.administrator = new Administrator();

            if (ServiceControl.Default.SelectedServices == null)
            {
                ServiceControl.Default.SelectedServices = new System.Collections.Specialized.StringCollection();
            }
        }

        /// <summary>
        /// Event handler for opening the context menu.
        /// </summary>
        /// <param name="sender">sender</param>
        /// <param name="e">EventArgs</param>
        /// <remarks>This method renews the content of the context menu each time it is executed.</remarks>
        void notifyIcon_Popup(object sender, EventArgs e)
        {
            // make the menu empty
            menu.MenuItems.Clear();
            //
            // Read the names of the controlled serviced from the application settings (SelectedServices).
            // For each service name create a MenuItem containing the service displayName and a subMenu
            // to start/stop the service.
            //
            // Get all services, i need the displayNames
            ServiceController[] services = ServiceController.GetServices();
            // create an array of ServiceController objects for the selectedServices
            ArrayList selectedServices = new ArrayList();
            // get the names of the controlled services from the settings
            System.Collections.Specialized.StringCollection sc = ServiceControl.Default.SelectedServices;
            System.Collections.Specialized.StringEnumerator enumerator = sc.GetEnumerator();
            while (enumerator.MoveNext())
            {
                String selectedService = enumerator.Current;
                ServiceController serviceController = findServiceController(services, selectedService);
                if (serviceController != null)
                {
                    selectedServices.Add(serviceController);
                }
            }
            // Create the menuItems, the number is the number of the fixed ones (exit, help...)
            // plus the variables
            MenuItem[] menuItems = new MenuItem[selectedServices.Count + 6];
            IEnumerator servicesEnumerator = selectedServices.GetEnumerator();
            int counter = 0;
            // The MenuItems for the services contains the displayName and a subMenu to control
            // the service status.
            while (servicesEnumerator.MoveNext())
            {
                ServiceController service = (ServiceController)servicesEnumerator.Current;
                MenuItem menuItem = new MenuItem(service.DisplayName);
                switch (service.Status)
                {
                    // if the service is already running, then create a disabled Start subMenuItem
                    // and an enabled Stop SubMenuItem
                    case ServiceControllerStatus.Running:
                        MenuItem subMenuItem;
                        subMenuItem = new MenuItem(this.messages.GetString("Controller_MenuItem_Start"));
                        subMenuItem.Enabled = false;
                        menuItem.MenuItems.Add(subMenuItem);
                        subMenuItem = new MenuItem(this.messages.GetString("Controller_MenuItem_Stop"));
                        // add a anonymous delegate method for stopping the service
                        subMenuItem.Click += delegate(object x, EventArgs ea)
                        {
                            ProgressDialog dialog = new ProgressDialog();
                            dialog.StopService(service);
                            if (this.administrator.Impersonate())
                                service.Stop();
                            else
                                dialog.Cancel();
                            this.administrator.Deimpersonate();
                        };
                        menuItem.MenuItems.Add(subMenuItem);
                        break;
                    // if the service is already stopped, then create a disabled Stop subMenuItem
                    // and an enabled Start SubMenuItem
                    case ServiceControllerStatus.Stopped:
                        subMenuItem = new MenuItem(this.messages.GetString("Controller_MenuItem_Start"));
                        subMenuItem.Click += delegate(object x, EventArgs ea)
                        {
                            // add a anonymous delegate method for starting the service
                            ProgressDialog dialog = new ProgressDialog();
                            dialog.StartService(service);
                            if (this.administrator.Impersonate())
                                service.Start();
                            else
                                dialog.Cancel();
                            this.administrator.Deimpersonate();
                        };
                        menuItem.MenuItems.Add(subMenuItem);
                        subMenuItem = new MenuItem(this.messages.GetString("Controller_MenuItem_Stop"));
                        subMenuItem.Enabled = false;
                        menuItem.MenuItems.Add(subMenuItem);
                        break;
                }
                menuItems[counter] = menuItem;
                counter++;
            }
            menuItems[counter] = new MenuItem("-");
            counter++;

            menuItems[counter] = new MenuItem(this.messages.GetString("Controller_MenuItem_Info"), menuItemInfo_Click);
            counter++;

            menuItems[counter] = new MenuItem(this.messages.GetString("Controller_MenuItem_Help"), menuItemHelp_Click);
            counter++;

            menuItems[counter] = new MenuItem(this.messages.GetString("Controller_MenuItem_Settings"), menuItemSettings_Click);
            counter++;

            menuItems[counter] = new MenuItem("-");
            counter++;

            menuItems[counter] = new MenuItem(this.messages.GetString("Controller_MenuItem_Exit"), menuItemExit_Click);
            menu.MenuItems.AddRange(menuItems);
        }
        /// <summary>
        /// Event handler for the Exit MenuItem.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void menuItemExit_Click(object sender, EventArgs e)
        {
            notifyicon.Dispose();
            Application.Exit();
        }
        /// <summary>
        /// Event handler for the Info MenuItem.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>Opens the AboutBox1</remarks>
        void menuItemInfo_Click(object sender, EventArgs e)
        {
            AboutBox1 aboutBox = new AboutBox1();
            aboutBox.ShowDialog();
        }
        /// <summary>
        /// Event handler for the Help MenuItem.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void menuItemHelp_Click(object sender, EventArgs e)
        {
            HelpDialog dlg = new HelpDialog();
            dlg.Show();
        }
        /// <summary>
        /// Event handler for the Settings MenuItem.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>Opens the ServicesSettingsForm and saves the result of the ServicesSettingsForm.</remarks>
        void menuItemSettings_Click(object sender, EventArgs e)
        {
            // Create a new ServicesSettingsForm and apply the former size.
            ServicesSettingsForm sf = new ServicesSettingsForm();
            Size size = ServiceControl.Default.SettingsFormSize;
            if( ( size != null ) && (size.Height>100)&(size.Width>100 ))
            {
                sf.Size = size;
            }
            // Set the available services
            ServiceController[] services = ServiceController.GetServices();
            sf.AvailableServices = services;
            ArrayList selectedServices = new ArrayList();
            System.Collections.Specialized.StringCollection sc = ServiceControl.Default.SelectedServices;
            System.Collections.Specialized.StringEnumerator enumerator = sc.GetEnumerator();
            while (enumerator.MoveNext())
            {
                String selectedService = enumerator.Current;
                ServiceController serviceController = findServiceController(services, selectedService);
                if (serviceController != null)
                {
                    selectedServices.Add(serviceController);
                }
            }
            // Set the selectes services (doesn't care that i have dupplettes in available services
            // and selected services (the Form handles this)
            sf.SelectedServices = selectedServices;
            // Open the dialog
            DialogResult result = sf.ShowDialog();
            if (result.Equals(DialogResult.OK))
            {
                // Save the controlled services names to settings
                selectedServices = sf.SelectedServices;
                this.controlledServices = selectedServices;
                ServiceControl.Default.SelectedServices.Clear();
                IEnumerator enumSelectedServices = selectedServices.GetEnumerator();
                while (enumSelectedServices.MoveNext())
                {
                    ServiceController serviceController = (ServiceController)enumSelectedServices.Current;
                    ServiceControl.Default.SelectedServices.Add(serviceController.ServiceName);
                }
            }
            ServiceControl.Default.SettingsFormSize = sf.Size;
            ServiceControl.Default.Save();
        }

        /// <summary>
        /// Finds the appropriate ServiceController for serviceName in services
        /// </summary>
        /// <param name="services">The array of ServicesController to search</param>
        /// <param name="serviceName">The name of the service to search for</param>
        /// <returns>The ServiceController with the name serviceName or null</returns>
        private ServiceController findServiceController(ServiceController[] services, String serviceName)
        {
            foreach (ServiceController sc in services)
            {
                if (sc.ServiceName == serviceName)
                {
                    return sc;
                }
            }
            return null;
        }
    }
}
