/*
    ServicesCtl is a programm to start and stop Services from the TrayBar.
    Copyright (C) 2007  Olaf Groeger

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    For any feedback contact me at servicesctl@olaf-groeger.de.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Resources;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace ServicesCtl
{
    /// <summary>
    /// This class encapsulates a dialog which more or less contains one
    /// progressbar. The class offers two public methods for showing the
    /// progress while starting/stopping a service.
    /// As soon as the service has been started/stopped the dialog closes.
    /// </summary>
    /// <see cref="StartService(ServiceController)"/>
    /// <see cref="StopService(ServiceController)"/>
    public partial class ProgressDialog : Form
    {
        /// <summary>
        /// Thread synchronization object. true when the 
        /// service is still being starting/stopping.
        /// </summary>
        private bool isRunning = false;
        /// <summary>
        /// The private System.Resources.ResourceManager used for internationalization.
        /// </summary>
        private ResourceManager messages = null;

        /// <summary>
        /// This delegate enables asynchronous calls for setting the text property of the labe1 TextBox control.
        /// </summary>
        /// <param name="text">The already translated text</param>
        delegate void SetTextCallback(string text);
        /// <summary>
        /// This delegate enables asynchronous calls for Closing the dialog.
        /// </summary>
        delegate void CloseDialogCallback();

        /// <summary>
        /// Creates a new instance.
        /// </summary>
        /// <remarks>Initialize the Form and creates a ResourceManager.</remarks>
        public ProgressDialog()
        {
            InitializeComponent();
            this.messages = new ResourceManager("ServicesCtl.Messages", Assembly.GetExecutingAssembly());
        }

        /// <summary>
        /// Monitors the start of a service.
        /// </summary>
        /// <param name="service">The service to monitor</param>
        /// <remarks>This method monitors the start of a service. It creates a Thread which continously checks
        /// the status of the service. As soon as the service has reached the running status, the dialog is being closed.</remarks>
        public void StartService(ServiceController service)
        {
            if (service == null)
                return;

            String strMessage = String.Format(this.messages.GetString("ProgressDialog_Starting"), service.DisplayName);

            this.Text = strMessage;

            lock (this)
            {
                this.isRunning = true;
            }
            Show();
            Thread thread = new Thread(new ParameterizedThreadStart(this.DoStartService));
            thread.Start(service);
        }

        /// <summary>
        /// Thread Start method. This method delegates to DoService(ServiceController service, bool start).
        /// </summary>
        /// <param name="serviceObject">The service to monitor</param>
        private void DoStartService(object serviceObject)
        {
            ServiceController service = (ServiceController)serviceObject;
            DoService(service, true);
        }

        /// <summary>
        /// Monitors the stop of a service.
        /// </summary>
        /// <param name="service">The service to monitor</param>
        /// <remarks>This method monitors the stop of a service. It creates a Thread which continously checks
        /// the status of the service. As soon as the service has reached the stopped status, the dialog is being closed.</remarks>
        public void StopService(ServiceController service)
        {
            if (service == null)
                return;

            this.Text = String.Format(this.messages.GetString("ProgressDialog_Stopping"), service.DisplayName);

            lock (this)
            {
                this.isRunning = true;
            }
            Show();
            Thread thread = new Thread(new ParameterizedThreadStart(this.DoStopService));
            thread.Start(service);
        }

        /// <summary>
        /// Thread Start method. This method delegates to DoService(ServiceController service, bool start).
        /// </summary>
        /// <param name="serviceObject">The service to monitor</param>
        private void DoStopService(object serviceObject)
        {
            ServiceController service = (ServiceController)serviceObject;
            DoService(service, false);
        }

        /// <summary>
        /// Checks the status of a service and closes the dialog when the final status has been reached.
        /// </summary>
        /// <param name="service">The service to monitor</param>
        /// <param name="start">true when a service start should be monitored, false when a service stop should be monitored</param>
        /// <remarks>This method refreshes the status of the service object continously and updates
        /// the status in the dialog according to the status. When the final status of the service has
        /// been reached (started or stopped) this method closes the dialog.</remarks>
        private void DoService(ServiceController service, bool start)
        {
            bool tempIsRunning = true;
            do
            {
                lock (this) { tempIsRunning = this.isRunning; }
                service.Refresh();
                ServiceControllerStatus status = service.Status;
                switch (status)
                {
                    case ServiceControllerStatus.ContinuePending:
                        SetLabelText(this.messages.GetString("ProgressDialog_ContinuePending"));
                        break;
                    case ServiceControllerStatus.Paused:
                        SetLabelText(this.messages.GetString("ProgressDialog_Paused"));
                        break;
                    case ServiceControllerStatus.PausePending:
                        SetLabelText(this.messages.GetString("ProgressDialog_PausePending"));
                        break;
                    case ServiceControllerStatus.Running:
                        SetLabelText(this.messages.GetString("ProgressDialog_Running"));
                        if(start)
                            tempIsRunning = false;
                        break;
                    case ServiceControllerStatus.StartPending:
                        SetLabelText(this.messages.GetString("ProgressDialog_StartPending"));
                        break;
                    case ServiceControllerStatus.Stopped:
                        SetLabelText(this.messages.GetString("ProgressDialog_Stopped"));
                        if(!start)
                            tempIsRunning = false;
                        break;
                    case ServiceControllerStatus.StopPending:
                        SetLabelText(this.messages.GetString("ProgressDialog_StopPending"));
                        break;
                }
                Thread.Sleep(0);
            } while (tempIsRunning);
            lock (this) { this.isRunning = false; }
            CloseDialog();
        }
        /// <summary>
        /// Closes the dialog and stops the monitor thread when running.
        /// </summary>
        public void Cancel()
        {
            lock (this) { this.isRunning = false; };
            CloseDialog();
        }

        /// <summary>
        /// Event handler for the cancel button
        /// </summary>
        /// <param name="sender">sender</param>
        /// <param name="e">EventArgs</param>
        private void button1_Click(object sender, EventArgs e)
        {
            Close();
            lock (this) { this.isRunning = false; }
        }

        /// <summary>
        /// Sets the Text of this.label1.
        /// </summary>
        /// <param name="text">The text to set</param>
        /// <remarks>This method handles calls from the UI and non-UI Threads.</remarks>
        private void SetLabelText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.label1.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetLabelText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.label1.Text = text;
            }
        }

        /// <summary>
        /// Closes the dialog.
        /// </summary>
        /// <remarks>This method handles calls from the UI and non-UI Threads.</remarks>
        private void CloseDialog()
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.InvokeRequired)
            {
                CloseDialogCallback d = new CloseDialogCallback(CloseDialog);
                this.Invoke(d);
            }
            else
            {
                Close();
            }
        }

    }
}