/*
    ServicesCtl is a programm to start and stop Services from the TrayBar.
    Copyright (C) 2007  Olaf Groeger

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    For any feedback contact me at servicesctl@olaf-groeger.de.
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.ServiceProcess;
using System.Text;
using System.Windows.Forms;

namespace ServicesCtl
{
    /// <summary>
    /// This Form offers the possibility to select services out of all available services.
    /// </summary>
    public partial class ServicesSettingsForm : Form
    {
        /// <summary>
        /// An Array of all local services.
        /// </summary>
        private ServiceController[] availableServices = new ServiceController[0];
        /// <summary>
        /// An array of the selected services. This array is just populated upon clicking the ok button.
        /// </summary>
        private ArrayList selectedServices = new ArrayList();
        /// <summary>
        /// A flag that the cancel button has been clicked.
        /// </summary>
        private bool canceled = false;

        /// <summary>
        /// Creates a new instance.
        /// </summary>
        public ServicesSettingsForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Event handler for the ok button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>This method populates the ServicesSettingsForm.selectedServices</remarks>
        private void buttonOk_Click(object sender, EventArgs e)
        {
            // Update selectedServices
            ArrayList result = new ArrayList();
            IEnumerator enumerator = this.listBox2.Items.GetEnumerator();
            while (enumerator.MoveNext())
            {
                result.Add(enumerator.Current);
            }
            this.selectedServices=result;

            this.canceled = false;
            // bye
            this.Close();
        }

        /// <summary>
        /// event handler for the cancel button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.canceled = true;
            this.Close();
        }

        /// <summary>
        /// An array of all local available services.
        /// </summary>
        public ServiceController[] AvailableServices
        {
            get
            {
                return this.availableServices;
            }
            set
            {
                this.availableServices = value;
                Array.Sort(this.availableServices, new ServiceControllerSorter());
            }
        }

        /// <summary>
        /// An Array of the selected Services
        /// </summary>
        public ArrayList SelectedServices
        {
            get
            {
                ArrayList result = new ArrayList();
                IEnumerator enumerator = this.listBox2.Items.GetEnumerator();
                while (enumerator.MoveNext())
                {
                    result.Add(enumerator.Current);
                }
                return result;
            }
            set
            {
                this.selectedServices = value;
                if (this.selectedServices != null)
                {
                    this.selectedServices.Sort(new ServiceControllerSorter());
                }
                else
                {
                    this.selectedServices = new ArrayList();
                }
            }
        }

        /// <summary>
        /// A flag that is set to true when the dialog has been closed by the cancel button.
        /// </summary>
        public bool Canceled
        {
            get
            {
                return this.canceled;
            }
        }

        /// <summary>
        /// Event handler for the add service button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAdd_Click(object sender, EventArgs e)
        {
            addToSelectedServices();
        }

        /// <summary>
        /// Event handler for the remove service button.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRemove_Click(object sender, EventArgs e)
        {
            removeFromSelectedServices();
        }

        /// <summary>
        /// Event handler for the load event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>This handler clears the listboxes and refills them. It removes invalid items from
        ///  the array of selected services</remarks>
        private void OnLoad(object sender, EventArgs e)
        {
            this.listBox1.Items.Clear();
            this.listBox2.Items.Clear();

            if (this.availableServices != null)
            {
                this.listBox1.Items.AddRange(this.availableServices);
            }
            if (this.selectedServices != null)
            {
                this.listBox2.Items.AddRange(this.selectedServices.ToArray());
                // now iterate over the available services and remove the
                // items of the selected services
                IEnumerator it = this.listBox2.Items.GetEnumerator();
                while (it.MoveNext())
                {
                    ServiceController service = (ServiceController)it.Current;
                    if (this.listBox1.Items.Contains(service))
                        this.listBox1.Items.Remove(service);
                    else
                        listBox2.Items.Remove(service);
                }
            }
        }

        /// <summary>
        /// Event handler for the double click in the listbox of available services.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAdd_DblClick(object sender, MouseEventArgs e)
        {
            addToSelectedServices();
        }

        /// <summary>
        /// Event handler for the double click in the listbox of selected services.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRemove_DblClick(object sender, MouseEventArgs e)
        {
            removeFromSelectedServices();
        }

        /// <summary>
        /// Adds the selected service in the listbox of available services to the listbox
        /// of controlled services.
        /// </summary>
        private void addToSelectedServices()
        {
            ServiceController selection = (ServiceController)listBox1.SelectedItem;
            if (selection == null)
                return;
            //this.selectedServices.Add(selection);
            listBox2.Items.Add(selection);
            
            listBox1.Items.Remove(selection);
        }
        /// <summary>
        /// Removes the selected service in the listbox of controlled services from the
        /// listbox and add it to the listbox of available services.
        /// </summary>
        private void removeFromSelectedServices()
        {
            ServiceController selection = (ServiceController)listBox2.SelectedItem;
            if (selection == null)
                return;
            listBox1.Items.Add(selection);
            //this.selectedServices.Remove(selection);
            listBox2.Items.Remove(selection);
        }
    }
    /// <summary>
    /// An IComparer implementation which compares ServiceController by its DisplayName
    /// </summary>
    public class ServiceControllerSorter : IComparer
    {

        // 
        /// <summary>
        /// Calls CaseInsensitiveComparer.Compare with the parameters reversed. 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        int IComparer.Compare(Object x, Object y)
        {
            ServiceController x1 = (ServiceController)x;
            ServiceController y1 = (ServiceController)y;
            return ((new CaseInsensitiveComparer()).Compare(x1.DisplayName, y1.DisplayName));
        }

    }
}